@@ -88,7 +88,7 @@ def createEDDM(screen, logfile, contrib, gaussdir, unres):
 
     def formchk(filename):
         filename = filename.split(fakeossep)[-1]
-        fchkpoint = filename.replace('.chk', '.fch')
+        fchkpoint = filename.replace('.chk', '.fck')
         report = "echo Formatting %s to %s" % (filename, fchkpoint)
         cmd = "%s%sformchk %s %s > tmp2.txt\n" % (envvar('G03DIR'), os.sep, filename,
                                       fchkpoint)
@@ -109,7 +109,7 @@ def createEDDM(screen, logfile, contrib, gaussdir, unres):
     if windows:
          syscmd = ['copy', 'del', 'type', 'move']
 
-    fchkpoint = glob.glob(os.path.join(gaussdir, "*.fch"))
+    fchkpoint = glob.glob(os.path.join(gaussdir, "*.fck")) + glob.glob(os.path.join(gaussdir, "*.fchk"))
     line = ""
     if len(fchkpoint) == 0:
         chkpoint = glob.glob(os.path.join(gaussdir, "*.chk"))
